unit im_childwin;

{*********************************************}
{  This unit is a part of ImageE              }
{  Copyright  2003-2004 R.Geurts             }
{  See Readme.txt for licence information     }
{*********************************************}

interface

uses Windows, Classes, Graphics, Forms, Controls, Menus,
   ExtCtrls, SysUtils, GR32, GR32_Image, gr32_layers,
   Messages, Dialogs, StdCtrls,math;

type
   TMDIChild = class(TForm)
    Imageview1: TImgView32;
    PopupMenu1: TPopupMenu;
    Paste21: TMenuItem;
      procedure FormClose(Sender: TObject; var Action: TCloseAction);
      procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
      procedure FormCreate(Sender: TObject);
    procedure Imageview1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
    procedure Imageview1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer; Layer: TCustomLayer);
    procedure DrawTheRect;
//    function NormalizeRect(R: TRect): TRect;
    procedure Select(R:Trect);
    procedure RBResizing(Sender: TObject;
  const OldLocation: TFloatRect; var NewLocation: TFloatRect;
  DragState: TDragState; Shift: TShiftState);
    procedure Imageview1Change(Sender: TObject);
    procedure MainFormCopy21Click(Sender: TObject);
    procedure Paste21Click(Sender: TObject);

   private

    { Private declarations }
   public
        RBLayer: TRubberbandLayer;
    { Public declarations }
      Changed: boolean;
   end;


var
  xorg,yorg,xmov,ymov:integer;
  Counter : Byte;
  CounterStart : Byte;
  Looper : LongInt;
  SelectionEnabled:boolean;
  AnnotateEnabled:boolean;

implementation

uses im_MainFrm, im_HistogramFRM;

{$R *.DFM}

////////////////////////////////////////////////////////////////////////////////////

procedure TMDIChild.FormCreate(Sender: TObject);
begin
 Changed := False;
end;

////////////////////////////////////////////////////////////////////////////////////

procedure TMDIChild.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   Action := caFree;
end;

////////////////////////////////////////////////////////////////////////////////////
// Imaged changed

procedure TMDIChild.ImageView1Change(Sender: TObject);
begin
{   Changed := true;}
end;

////////////////////////////////////////////////////////////////////////////////////

procedure TMDIChild.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
   CanClose := true;
   if Changed then
      case MessageDlg('Save Changes to ' + Caption + '?', mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
         mrYes:
          //  frmMain.FileSaveItemClick(self);
         //if mrCancel:exit;
         CanClose := false;
      end;
end;

////////////////////////////////////////////////////////////////////////////////////

procedure TMDIChild.Select(R:Trect);
begin
   if selectionenabled=false then exit;
   if selectionactive=false then
   begin
   selectionactive:=true;
RBLayer := TRubberBandLayer.Create(imageview1.Layers);
   end;
with RBLayer do begin
    LayerOptions := LOB_VISIBLE or LOB_MOUSE_EVENTS;
    OnResizing := RBResizing;
    Scaled := True;
    Visible := true;
end;
  with RBLayer do
    begin
      Location := FloatRect(R);
      Visible := True;
      BringToFront;
    end;
  imageview1.Changed;
end;

procedure TMDIChild.RBResizing(Sender: TObject;
  const OldLocation: TFloatRect; var NewLocation: TFloatRect;
  DragState: TDragState; Shift: TShiftState);
var
{  CanCrop: Boolean;}
  R: TRect;
  RectChanged: Boolean;
begin
  selectionenabled:=false;
  RectChanged := False;
  R:= MakeRect(NewLocation);
  // stop rubber band going outside bitmap area
  if R.Left < 0 then begin
    R.Left := 0;
    RectChanged := True;
  end;
  if R.Top < 0 then begin
    R.Top := 0;
    RectChanged := True;
  end;
  with imageview1.Bitmap do begin
    if R.Right > Width then begin
      R.Right := Width;
      RectChanged := True;
    end;
    if R.Bottom > Height then begin
      R.Bottom := Height;
      RectChanged := True;
    end;
{    CanCrop := (R.Left <> 0) or (R.Top <> 0) or (R.Right <> Width)
      or (R.Bottom <> Height);}
  end;
  if RectChanged then NewLocation := FloatRect(R);
end;

function NormalizeRect(R: TRect): TRect;
begin
  // This routine normalizes a rectangle. It makes sure that the Left,Top
  // coords are always above and to the left of the Bottom,Right coords.
  with R do
    if Left > Right then
      if Top > Bottom then
        Result := Rect(Right,Bottom,Left,Top)
      else
        Result := Rect(Right,Top,Left,Bottom)
    else
      if Top > Bottom then
        Result := Rect(Left,Bottom,Right,Top)
      else
        Result := Rect(Left,Top,Right,Bottom);
end;

procedure TMDIChild.DrawTheRect;
var
R:trect;
begin
  Counter := CounterStart;
 R:=rect(xorg,yorg,xmov,ymov);
 R:=normalizerect(R);
with R do
begin
 if left<0 then left:=0;
 if right>imageview1.bitmap.width then right:=imageview1.bitmap.width;
 if top<0 then top:=0;
 if bottom>imageview1.bitmap.height then bottom:=imageview1.bitmap.height;
end;
 select(R);
end;

procedure TMDIChild.Imageview1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer;
  Layer: TCustomLayer);
var np1,np2:tpoint;
xr,yr:integer;
begin
np1.X:=x;
np1.y:=y;
np2:=imageview1.ControlToBitmap(np1);
xr:=np2.X;
yr:=np2.y;

if imageview1.Cursor=crdefault then selectionenabled:=true;
if selectionenabled=false then exit;
Xorg := Xr; Yorg := Yr; Xmov := Xr; Ymov := Yr;
end;

procedure TMDIChild.Imageview1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
var np1,np2:tpoint;
xr,yr:integer;
begin
np1.X:=x;
np1.y:=y;
np2:=imageview1.ControlToBitmap(np1);
xr:=np2.X;
yr:=np2.y;
mainform.StatusBar.Panels[0].Text:='x:'+floattostr(xr)+' - y:'+floattostr(yr);

//if imageview1.Cursor=crdefault then selectionenabled:=true;
if selectionenabled=false then exit;
// if shift=false then begin
   if ssLeft in Shift then
    begin
//      RemoveTheRect;         // Erase any existing rectangle
      Xmov := Xr; Ymov := Yr;      // Save the new corner where the mouse is
      DrawTheRect;           // Draw the Rect now... don't wait for the timer!
    end;
// end;
end;

procedure TMDIChild.MainFormCopy21Click(Sender: TObject);
begin
mainform.Copy1click(self);
end;

procedure TMDIChild.Paste21Click(Sender: TObject);
begin
mainform.Paste1Click(self);
end;

end.


