unit im_toolfrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls,gr32, StdCtrls;

type
  TToolForm = class(TForm)
    Image1: TImage;
    Label1: TLabel;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Button1Click(Sender: TObject);
   private
    OriginalRangeLeft:  INTEGER;
    ScaleFactor      :  DOUBLE;
    StretchFactor    :  DOUBLE;
    TailPercentLeft  :  DOUBLE;
    TailPercentRight :  DOUBLE;
    ShowDesignHeight :  INTEGER;
    ShowDesignWidth  :  INTEGER;
    procedure CreateHistoStretchBitmap(dst:TBitmap32);
    procedure CheckBox1Click(Sender: TObject);

  public
    dst:tbitmap32;
    src:tbitmap32;
    PROCEDURE UpdateDisplay;
  end;

var
  Toolform: TToolform;
  stretchenabled:boolean;
implementation

uses im_histogramlib;
{$R *.dfm}

PROCEDURE TToolform.UpdateDisplay;
  VAR
    OriginalHistogram :  THistogram;
    OriginalRangeRight:  INTEGER;
    OriginalRange     :  INTEGER;
    StretchedBitmap   :  TBitmap32;
    StretchedHistogram:  THistogram;
    StretchedRange    :  INTEGER;

  PROCEDURE ShowHistogram(CONST Histogram:  THistogram;
                          CONST Image:  TImage;
                          CONST LabelStats:  TLabel);
    CONST
      clSkyBlue    = TColor($F0CAA6);   // RGB:  166 202 240
    VAR
      Kurtosis         :  DOUBLE;
      Maximum          :  BYTE;
      Mean             :  DOUBLE;
      Median           :  BYTE;
      Minimum          :  BYTE;
      Mode             :  BYTE;
      N                :  INTEGER;
      Skewness         :  DOUBLE;
      StandardDeviation:  DOUBLE;
  BEGIN
    Image.Canvas.Brush.Color := clSkyBlue;
    Image.Canvas.FillRect(Rect(0, 0, Image.Width, Image.Height));

    Histogram.Draw(Image.Canvas);
    Histogram.GetStatistics(N, Minimum, Maximum,
       Mode, Median,
       Mean, StandardDeviation,
       Skewness, Kurtosis);


    LabelStats.Caption := 'Statistics:  ' +
      Format('%d / %d / %d / %d / %.1f / %.1f / %.1f',
      [Minimum,Maximum,Mode,Median,Mean,StandardDeviation, Kurtosis-3.0])
  END {UpdateHistogram};

BEGIN
  Screen.Cursor := crHourGlass;
  TRY
    OriginalHistogram := THistogram.Create;
    TRY
      // Ask for cpRed, but cpGreen or cpBlue will give identical results
      // since R = G = B for shades of gray
      GetHistogram(cpRed, dst, OriginalHistogram);
      ShowHistogram(OriginalHistogram, Image1, label1);

      OriginalRangeLeft  := OriginalHistogram.GetPercentileLevel(TailPercentLeft);
      OriginalRangeRight := OriginalHistogram.GetPercentileLevel(TailPercentRight);
      OriginalRange      := OriginalRangeRight - OriginalRangeLeft;

      // Normally, StretchFactor is 1.00 and RangeStretched = 255 (full width)
      StretchedRange := OriginalRange + ROUND( StretchFactor*(255 - OriginalRange) );

      IF   OriginalRange = 0.0
      THEN ScaleFactor := 1.0
      ELSE ScaleFactor := StretchedRange / OriginalRange;

      if stretchenabled=true then
      begin
      createhistostretchbitmap(dst);
      GetHistogram(cpRed, dst, OriginalHistogram);
      ShowHistogram(OriginalHistogram, Image1, label1);
      end;
    FINALLY
      OriginalHistogram.Free
    END
  FINALLY
    Screen.Cursor := crDefault
  END
END {UpdateDisplay};

procedure TToolform.CreateHistoStretchBitmap(dst:TBitmap32);
  VAR
    i,j               :  INTEGER;
    Intensity         :  INTEGER;
    OriginalRow       :  Pcolor32array;
    StretchedRow      :  Pcolor32array;
    h1,h2,h3:integer;
    SS:Pcolor32;
    ri,gi,bi:integer;
begin
    h1:=$0000FF; //0
    h2:=$00FF00; //1
    h3:=$FF0000; //2
  // Create stretched bitmap
  SS := @dst.Bits[0];

  FOR j := 0 TO dst.Height-1 DO
  BEGIN
    OriginalRow  := dst.Scanline[j];
    FOR i := 0 TO dst.Width-1 DO
    BEGIN
      // Use the R component, but R = G = B for shades of gray
      bi:= originalrow[i] and h1;
      gi:= (originalrow[i] and h2) shr 8;
      ri:= (originalrow[i] and h3) shr 16;
      // Calculate new intensity level
      ri := ROUND(ScaleFactor*(rI - OriginalRangeLeft) );
      gi := ROUND(ScaleFactor*(gI - OriginalRangeLeft) );
      bi := ROUND(ScaleFactor*(bI - OriginalRangeLeft) );

      IF   rI < 0
      THEN rI := 0
      ELSE
        IF   ri > 255
        THEN rI := 255;
      IF   gI < 0
      THEN gI := 0
      ELSE
        IF   gi > 255
        THEN gI := 255;
      IF   bI < 0
      THEN bI := 0
      ELSE
        IF   bi > 255
        THEN bI := 255;

//      gi:=ri; bi:=ri;

      SS^ := $FF000000 + ri shl 16 + gi shl 8 + bi;
      inc(ss);
    END
    END
END {CreateHistoStretchBitamp};


procedure TToolform.FormCreate(Sender: TObject);
begin
  dst:=tbitmap32.create;
  src:=tbitmap32.create;
  TailPercentLeft  :=  1.0;
  TailPercentRight := 99.0;
  StretchFactor    :=  1.0;
end;

procedure TToolform.CheckBox1Click(Sender: TObject);
begin
dst.Assign(src);
stretchenabled:=false;
// checkbox1.Checked=true then stretchenabled:=true;
updatedisplay;
end;

procedure TToolform.FormShow(Sender: TObject);
begin
src.Assign(dst);
//checkbox1click(self);
end;

procedure TToolForm.Button1Click(Sender: TObject);
begin
updatedisplay;
end;

end.
